#!/usr/bin/perl -w
#
# efetch-wrapper 0.1b
# A simple script to extend wget to download more types of uri's.  Specifically
# for Gentoo's emerge process, but can be extended for other applications.
#
#    Copyright (C) 2004,  Iain Buchanan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# usage:
# FETCHCOMMAND="emerge-wrapper.pl \${DISTDIR} \${URI}"
# ./emerge-wrapper.pl /usr/portage/distfiles url://user:pass@host/path/to/file
#
# requires:
# Expect (which requires other perl modules, but I'll leave that up to you)
#

use strict;
use IO::File;
# use IPC::Open3; # for pipe (open2)
use Expect;

# this is the default FETCHCOMMAND (I added -c because wget doesn't care so
# there's not point in having a different fetch and resume command)
my $wget = "/usr/bin/wget -c -t 5 --passive-ftp -P $ARGV[0]"; # $ARGV[1]"
my $rsync = "/usr/bin/rsync -avzP";
my $get = $wget;
my $mode = 0600;
my $filename = '';
my $pass  = '';
my $proto = '';
my $user = '';

if ($ARGV[1] =~ /^(ftp|http|rsync|ssh):\/\/(.+?:.*\@)?(.+?)$/) {
   $proto = $1;
   my $rest  = $3;
   $user  = $2;
   if ($2 && $user =~ /^([^:]+)\@$/) {      # user@host
      $user = $1;
   }
   elsif ($2 && $user =~ /^(.+):(.+)\@$/) { # user:pass@host
      $user = $1;
      $pass = $2;
   }
   else {
      $user = $pass = '';
   }

   # print "getting $rest using $proto with $user:$pass\n";
   if ($proto eq 'http') {
      if ($user ne '') { $get .= " --http-user=$user"; }
      if ($pass ne '') { $get .= " --http-passwd=\"$pass\""; }
      $get .= " $proto://$rest";
   }
   elsif ($proto eq 'ftp') {

      # notes about ftp:
      # 1. you can't use some characters in your password, you could try
      # escaping them, or encoding them, but I didn't figure it out.
      # 2. if you need to get files from the root directory of the ftp server,
      # you may have to use // in the path.  eg:
      # ftp://user:pass@server//usr/portage/distfiles.  Make sure when you log
      # in with this user name and password you're not chrooted to your home
      # (some ftp servers do this for all non-anonymous log ins)
      # 3. ****************** ftp is insecure! ****************** people can
      # look at make.conf, ps, and sniff packets to see your password.

      $get .= " $proto://";
      if ($user ne '') {
         $get .= "$user";
         if ($pass ne '') {
            $get .= ":$pass\@";
         }
         else {
            $get .= '@';
         }
      }
      $get .= $rest;
   }
   elsif ($proto eq 'rsync') {

      # rsync / ssh is a little more tricky.  In the interests of security, we
      # don't want the password in an environment variable _or_ a command line
      # arguement, so it has to go in a root-owned read-only file.  However,
      # this only works when you don't use "-e ssh".  Also, some people like to
      # run multiple emerges at once, so the file should have a unique name.
      # The name of the package we're getting + timestamp perhaps?

      $filename = $rest . '.' . time;
      $filename =~ s/[\/: ;]+/./g;
      $filename = $ENV{'PORTAGE_TMPDIR'} . "/.$filename";
      open (RP, ">$filename") or die "couldn't open '$filename': $!\n";
      if (! chown 0, 0, "$filename")  { die "couldn't change ownership of '$filename'\n" };
      if (! chmod $mode, "$filename") { die "couldn't change permissions of '$filename'\n" };
      print RP $pass;
      close RP;
      $get = "$rsync --password-file=$filename rsync://$user\@$rest $ARGV[0]";
   }
   elsif ($proto eq 'ssh') {
      $get = "$rsync -e ssh $user\@$rest $ARGV[0]";
   }
}
else {
   print "unknown uri, falling back to wget:\n$ARGV[1]\n";
   $get .= " $ARGV[1]";
}

# print "\nuri: $ARGV[1]\ncommand: $get\n\n\n";
if ($proto ne 'ssh') {
   `$get`;
}

if ($proto eq 'rsync') {
   if (! unlink $filename) {
      print STDERR "couldn't unlink '$filename', you should probably delete it\n";
   }
}
elsif ($proto eq 'ssh') {
   my $exp = Expect->spawn($get) or die "Cannot spawn $get: $!\n";
   $exp->log_stdout(1);
   # $exp->log_file(*STDOUT);
   my $res = $exp->expect(10, '-re', '(P|p)assword:\s');
   if ($res) {
      print "*password sent*\n";
      $exp->send("$pass\n");
   }
   else {
      die "password prompt never appeared.  Try logging in by hand\n";
   }

   my $finished = 0;
   while (!$finished) {
      my @res = $exp->expect(1, '-re', '(P|p)assword:\s');
      if (!defined ($res[1])) { # no error, ie it matched
         print STDERR "login failed! Please check the password. ";
         $finished = 1;
      }
      if ($res[1] =~ /^3:/) { # child died
         # print "rsync finished";
         $finished = 1;
      }
      if (!$finished)  {
         # print "*: @res\n";
         # sleep 1;
         ;
      }
   }
   print "efetch finishing... please wait\n";
   $exp->soft_close();
   print "efetch done\n";
}
